// This file is part of the happyDomain (R) project.
// Copyright (c) 2022-2024 happyDomain
// Authors: Pierre-Olivier Mercier, et al.
//
// This program is offered under a commercial and under the AGPL license.
// For commercial licensing, contact us at <contact@happydomain.org>.
//
// For AGPL licensing:
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Affero General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Affero General Public License for more details.
//
// You should have received a copy of the GNU Affero General Public License
// along with this program.  If not, see <https://www.gnu.org/licenses/>.

import { handleApiResponse } from '$lib/errors';
import type { Domain, DomainInList } from '$lib/model/domain';
import type { ServiceCombined, ServiceMeta } from '$lib/model/service';
import type { ServiceRecord, Zone, ZoneMeta } from '$lib/model/zone';

export async function getZone(domain: Domain | DomainInList, id: string): Promise<Zone> {
    const dnid = encodeURIComponent(domain.id);
    id = encodeURIComponent(id);
    const res = await fetch(`/api/domains/${dnid}/zone/${id}`, {headers: {'Accept': 'application/json'}});
    return await handleApiResponse<Zone>(res);
}

export async function viewZone(domain: Domain | DomainInList, id: string): Promise<string> {
    const dnid = encodeURIComponent(domain.id);
    id = encodeURIComponent(id);
    const res = await fetch(`/api/domains/${dnid}/zone/${id}/view`, {
        method: 'POST',
        headers: {'Accept': 'application/json'}
    });
    return await handleApiResponse<string>(res);
}

export async function retrieveZone(domain: Domain | DomainInList): Promise<ZoneMeta> {
    const dnid = encodeURIComponent(domain.id);
    const res = await fetch(`/api/domains/${dnid}/retrieve_zone`, {
        method: 'POST',
        headers: {'Accept': 'application/json'}
    });
    return await handleApiResponse<ZoneMeta>(res);
}

export async function applyZone(domain: Domain | DomainInList, id: string, wantedCorrections: Array<string>, commitMessage: string): Promise<ZoneMeta> {
    const dnid = encodeURIComponent(domain.id);
    id = encodeURIComponent(id);
    const res = await fetch(`/api/domains/${dnid}/zone/${id}/apply_changes`, {
        method: 'POST',
        headers: {'Accept': 'application/json'},
        body: JSON.stringify({wantedCorrections, commitMessage}),
    });
    return await handleApiResponse<ZoneMeta>(res);
}

export async function importZone(domain: Domain | DomainInList, id: string, file: any): Promise<ZoneMeta> {
    const dnid = encodeURIComponent(domain.id);
    id = encodeURIComponent(id);

    const formData = new FormData();
    formData.append("zone", file);

    const res = await fetch(`/api/domains/${dnid}/zone/${id}/import`, {
        method: 'POST',
        headers: {'Accept': 'application/json'},
        body: formData,
    });
    return await handleApiResponse<ZoneMeta>(res);
}

export async function diffZone(domain: Domain | DomainInList, id1: string, id2: string): Promise<Array<string>> {
    const dnid = encodeURIComponent(domain.id);
    id1 = encodeURIComponent(id1);
    id2 = encodeURIComponent(id2);
    const res = await fetch(`/api/domains/${dnid}/diff_zones/${id1}/${id2}`, {
        method: 'POST',
        headers: {'Accept': 'application/json'}
    });
    return await handleApiResponse<Array<string>>(res);
}

export async function addZoneService(domain: Domain | DomainInList, id: string, service: ServiceCombined): Promise<Zone> {
    let subdomain = service._domain;
    if (subdomain === '') subdomain = '@';

    const dnid = encodeURIComponent(domain.id);
    id = encodeURIComponent(id);
    subdomain = encodeURIComponent(subdomain);

    const res = await fetch(`/api/domains/${dnid}/zone/${id}/${subdomain}/services`, {
        method: 'POST',
        headers: {'Accept': 'application/json'},
        body: JSON.stringify(service)
    });
    return await handleApiResponse<Zone>(res);
}

export async function updateZoneService(domain: Domain | DomainInList, id: string, service: ServiceCombined): Promise<Zone> {
    const dnid = encodeURIComponent(domain.id);
    id = encodeURIComponent(id);

    const res = await fetch(`/api/domains/${dnid}/zone/${id}`, {
        method: 'PATCH',
        headers: {'Accept': 'application/json'},
        body: JSON.stringify(service),
    });
    return await handleApiResponse<Zone>(res);
}

export async function deleteZoneService(domain: Domain | DomainInList, id: string, service: ServiceMeta): Promise<Zone> {
    let subdomain = service._domain;
    if (subdomain === '') subdomain = '@';

    const dnid = encodeURIComponent(domain.id);
    id = encodeURIComponent(id);
    subdomain = encodeURIComponent(subdomain);
    const svcid = service._id?encodeURIComponent(service._id):undefined;

    const res = await fetch(`/api/domains/${dnid}/zone/${id}/${subdomain}/services/${svcid}`, {
        method: 'DELETE',
        headers: {'Accept': 'application/json'}
    });
    return await handleApiResponse<Zone>(res);
}

export async function getServiceRecords(domain: Domain | DomainInList, id: string, service: ServiceMeta): Promise<Array<ServiceRecord>> {
    let subdomain = service._domain;
    if (subdomain === '') subdomain = '@';

    const dnid = encodeURIComponent(domain.id);
    id = encodeURIComponent(id);
    const svcid = service._id?encodeURIComponent(service._id):undefined;

    const res = await fetch(`/api/domains/${dnid}/zone/${id}/${subdomain}/services/${svcid}/records`, {
        headers: {'Accept': 'application/json'}
    });
    return await handleApiResponse<Array<ServiceRecord>>(res);
}
